import { SceneGridItemLike, SceneGridRow, VizPanel } from '@grafana/scenes';

import { DashboardGridItem } from '../../../features/dashboard-scene/scene/DashboardGridItem';

import { A4_HEIGHT, A4_WIDTH, FOOTER } from './ReportGridRenderer';
import { buildGridLayout } from './utils';

describe('buildGridLayout', () => {
  test('layout with large panels and rows', () => {
    const dashboardItems: SceneGridItemLike[] = [
      new DashboardGridItem({
        key: 'gridItem-1',
        x: 0,
        y: 0,
        height: 4,
        width: 24,
        body: new VizPanel({
          title: 'Top panel',
        }),
      }),
      new SceneGridRow({
        key: 'gridRow-1',
        title: 'First row',
        y: 4,
        children: [
          new DashboardGridItem({
            key: 'gridItem-2',
            x: 0,
            y: 5,
            height: 6,
            width: 12,
            body: new VizPanel({
              title: 'Panel A',
            }),
          }),
          new DashboardGridItem({
            key: 'gridItem-3',
            x: 12,
            y: 5,
            height: 6,
            width: 12,
            body: new VizPanel({
              title: 'Panel B',
            }),
          }),
        ],
      }),
      new SceneGridRow({
        key: 'gridRow-2',
        title: 'Second row',
        y: 11,
        children: [
          new DashboardGridItem({
            key: 'gridItem-4',
            x: 0,
            y: 12,
            height: 17,
            width: 24,
            body: new VizPanel({
              title: 'Large panel after row',
            }),
          }),
          new DashboardGridItem({
            key: 'gridItem-5',
            x: 0,
            y: 29,
            height: 24,
            width: 24,
            body: new VizPanel({
              title: 'Large panel after panel',
            }),
          }),
        ],
      }),
    ];
    const scale = 1.5;
    const pages = buildGridLayout(dashboardItems, scale, (A4_HEIGHT - 110 - FOOTER) * scale);

    expect(pages.length).toBe(2);
    expect(pages[0].items.length).toBe(6);
    expect(pages[1].items.length).toBe(1);
  });

  test('panels with the same Y position remain in the same page', () => {
    const dashboardItems: SceneGridItemLike[] = [
      new DashboardGridItem({
        key: 'gridItem-1',
        x: 0,
        y: 0,
        height: 6,
        width: 5,
        body: new VizPanel({
          title: 'Top panel A',
        }),
      }),
      new DashboardGridItem({
        key: 'gridItem-2',
        x: 12,
        y: 0,
        height: 10,
        width: 10,
        body: new VizPanel({
          title: 'Top panel B',
        }),
      }),
      new DashboardGridItem({
        key: 'gridItem-3',
        x: 0,
        y: 10,
        height: 6,
        width: 2,
        body: new VizPanel({
          title: 'Panel with Y 5',
        }),
      }),
      new DashboardGridItem({
        key: 'gridItem-4',
        x: 2,
        y: 10,
        height: 20,
        width: 12,
        body: new VizPanel({
          title: 'Sibling panel with same Y but big height',
        }),
      }),
      new DashboardGridItem({
        key: 'gridItem-5',
        x: 0,
        y: 30,
        height: 10,
        width: 12,
        body: new VizPanel({
          title: 'Panel in different page',
        }),
      }),
      new DashboardGridItem({
        key: 'gridItem-6',
        x: 4,
        y: 30,
        height: 10,
        width: 12,
        body: new VizPanel({
          title: 'Panel in different page',
        }),
      }),
    ];
    const scale = 1;
    const pages = buildGridLayout(dashboardItems, scale, (A4_WIDTH - 92 - FOOTER) * scale);

    expect(pages.length).toBe(3);
    expect(pages[0].items.length).toBe(2);
    expect(pages[1].items.length).toBe(2);
    expect(pages[2].items.length).toBe(2);
  });

  test('panels with the same Y position remain in the same page - second case', () => {
    const dashboardItems: SceneGridItemLike[] = [
      new DashboardGridItem({
        key: 'gridItem-1',
        x: 0,
        y: 0,
        height: 10,
        width: 12,
        body: new VizPanel({
          title: 'Top panel A',
        }),
      }),
      new DashboardGridItem({
        key: 'gridItem-2',
        x: 12,
        y: 0,
        height: 10,
        width: 12,
        body: new VizPanel({
          title: 'Top panel B',
        }),
      }),
      new DashboardGridItem({
        key: 'gridItem-3',
        x: 0,
        y: 10,
        height: 6,
        width: 2,
        body: new VizPanel({
          title: 'Panel with Y 5 and small height',
        }),
      }),
      new DashboardGridItem({
        key: 'gridItem-4',
        x: 2,
        y: 10,
        height: 16,
        width: 12,
        body: new VizPanel({
          title: 'Sibling panel with same Y but big height',
        }),
      }),
      new DashboardGridItem({
        key: 'gridItem-5',
        x: 14,
        y: 10,
        height: 16,
        width: 10,
        body: new VizPanel({
          title: 'Sibling panel with same Y but big height',
        }),
      }),
      new DashboardGridItem({
        key: 'gridItem-6',
        x: 0,
        y: 16,
        height: 10,
        width: 2,
        body: new VizPanel({
          title: 'Panel',
        }),
      }),
    ];
    const scale = 1;
    const pages = buildGridLayout(dashboardItems, scale, (A4_WIDTH - 92 - FOOTER) * scale);

    expect(pages.length).toBe(2);
    expect(pages[0].items.length).toBe(2);
    expect(pages[1].items.length).toBe(4);
  });
});
